@@ -13,13 +13,15 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import argparse
+import copy
 import mock
 from mock import call
 import testtools
 
+from osc_lib.cli import format_columns
 from osc_lib import exceptions
 from osc_lib.tests import utils as tests_utils
-from osc_lib import utils
 
 from neutronclient.osc.v2.trunk import network_trunk as trunk
 from neutronclient.osc.v2 import utils as v2_utils
@@ -48,14 +50,14 @@ class TestCreateNetworkTrunk(test_fakes.TestNeutronClientOSCV2):
 
     def get_data(self):
         return (
-            v2_utils.format_admin_state(self._trunk['admin_state_up']),
+            v2_utils.AdminStateColumn(self._trunk['admin_state_up']),
             self._trunk['description'],
             self._trunk['id'],
             self._trunk['name'],
             self._trunk['port_id'],
             self._trunk['project_id'],
             self._trunk['status'],
-            utils.format_list_of_dicts(self._trunk['sub_ports']),
+            format_columns.ListDictColumn(self._trunk['sub_ports']),
         )
 
     def setUp(self):
@@ -95,7 +97,7 @@ class TestCreateNetworkTrunk(test_fakes.TestNeutronClientOSCV2):
                           'port_id': self._trunk['port_id']}
         })
         self.assertEqual(self.columns, columns)
-        self.assertEqual(self.data, data)
+        self.assertItemEqual(self.data, data)
 
     def test_create_full_options(self):
         self._trunk['description'] = 'foo description'
@@ -135,7 +137,7 @@ class TestCreateNetworkTrunk(test_fakes.TestNeutronClientOSCV2):
                           'port_id': self._trunk['port_id']}
         })
         self.assertEqual(self.columns, columns)
-        self.assertEqual(self.data, data)
+        self.assertItemEqual(self.data, data)
 
     def test_create_trunk_with_subport_invalid_segmentation_id_fail(self):
         subport = self._trunk['sub_ports'][0]
@@ -162,6 +164,56 @@ class TestCreateNetworkTrunk(test_fakes.TestNeutronClientOSCV2):
             self.assertEqual("Segmentation-id 'boom' is not an integer",
                              str(e))
 
+    def test_create_network_trunk_subports_without_optional_keys(self):
+        subport = copy.copy(self._trunk['sub_ports'][0])
+        # Pop out the segmentation-id and segmentation-type
+        subport.pop('segmentation_type')
+        subport.pop('segmentation_id')
+        arglist = [
+            '--parent-port', self._trunk['port_id'],
+            '--subport', 'port=%(port)s' % {'port': subport['port_id']},
+            self._trunk['name'],
+        ]
+        verifylist = [
+            ('name', self._trunk['name']),
+            ('parent_port', self._trunk['port_id']),
+            ('add_subports', [{
+                'port': subport['port_id']}]),
+        ]
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        columns, data = (self.cmd.take_action(parsed_args))
+
+        self.neutronclient.create_trunk.assert_called_once_with({
+            trunk.TRUNK: {'name': self._trunk['name'],
+                          'admin_state_up': True,
+                          'sub_ports': [subport],
+                          'port_id': self._trunk['port_id']}
+        })
+        self.assertEqual(self.columns, columns)
+        self.assertItemEqual(self.data, data)
+
+    def test_create_network_trunk_subports_without_required_key_fail(self):
+        subport = self._trunk['sub_ports'][0]
+        arglist = [
+            '--parent-port', self._trunk['port_id'],
+            '--subport', 'segmentation-type=%(seg_type)s,'
+            'segmentation-id=%(seg_id)s' % {
+                'seg_id': subport['segmentation_id'],
+                'seg_type': subport['segmentation_type']},
+            self._trunk['name'],
+        ]
+        verifylist = [
+            ('name', self._trunk['name']),
+            ('parent_port', self._trunk['port_id']),
+            ('add_subports', [{
+                'segmentation-id': str(subport['segmentation_id']),
+                'segmentation-type': subport['segmentation_type']}]),
+        ]
+
+        with testtools.ExpectedException(argparse.ArgumentTypeError):
+            self.check_parser(self.cmd, arglist, verifylist)
+
 
 class TestDeleteNetworkTrunk(test_fakes.TestNeutronClientOSCV2):
     # The trunk to be deleted.
@@ -249,14 +301,14 @@ class TestShowNetworkTrunk(test_fakes.TestNeutronClientOSCV2):
         'sub_ports',
     )
     data = (
-        v2_utils.format_admin_state(_trunk['admin_state_up']),
+        v2_utils.AdminStateColumn(_trunk['admin_state_up']),
         _trunk['description'],
         _trunk['id'],
         _trunk['name'],
         _trunk['port_id'],
         _trunk['project_id'],
         _trunk['status'],
-        utils.format_list_of_dicts(_trunk['sub_ports']),
+        format_columns.ListDictColumn(_trunk['sub_ports']),
     )
 
     def setUp(self):
@@ -291,7 +343,7 @@ class TestShowNetworkTrunk(test_fakes.TestNeutronClientOSCV2):
         self.neutronclient.show_trunk.assert_called_once_with(
             self._trunk['id'])
         self.assertEqual(self.columns, columns)
-        self.assertEqual(self.data, data)
+        self.assertItemEqual(self.data, data)
 
 
 class TestListNetworkTrunk(test_fakes.TestNeutronClientOSCV2):
@@ -328,7 +380,7 @@ class TestListNetworkTrunk(test_fakes.TestNeutronClientOSCV2):
             t['port_id'],
             t['description'],
             t['status'],
-            v2_utils.format_admin_state(t['admin_state_up']),
+            v2_utils.AdminStateColumn(t['admin_state_up']),
             '2001-01-01 00:00:00',
             '2001-01-01 00:00:00',
         ))
@@ -352,7 +404,7 @@ class TestListNetworkTrunk(test_fakes.TestNeutronClientOSCV2):
 
         self.neutronclient.list_trunks.assert_called_once_with()
         self.assertEqual(self.columns, columns)
-        self.assertEqual(self.data, list(data))
+        self.assertListItemEqual(self.data, list(data))
 
     def test_trunk_list_long(self):
         arglist = [
@@ -367,7 +419,7 @@ class TestListNetworkTrunk(test_fakes.TestNeutronClientOSCV2):
 
         self.neutronclient.list_trunks.assert_called_once_with()
         self.assertEqual(self.columns_long, columns)
-        self.assertEqual(self.data_long, list(data))
+        self.assertListItemEqual(self.data_long, list(data))
 
 
 class TestSetNetworkTrunk(test_fakes.TestNeutronClientOSCV2):
@@ -385,14 +437,14 @@ class TestSetNetworkTrunk(test_fakes.TestNeutronClientOSCV2):
         'sub_ports',
     )
     data = (
-        v2_utils.format_admin_state(_trunk['admin_state_up']),
+        v2_utils.AdminStateColumn(_trunk['admin_state_up']),
         _trunk['id'],
         _trunk['name'],
         _trunk['description'],
         _trunk['port_id'],
         _trunk['project_id'],
         _trunk['status'],
-        utils.format_list_of_dicts(_trunk['sub_ports']),
+        format_columns.ListDictColumn(_trunk['sub_ports']),
     )
 
     def setUp(self):
@@ -511,6 +563,50 @@ class TestSetNetworkTrunk(test_fakes.TestNeutronClientOSCV2):
         )
         self.assertIsNone(result)
 
+    def test_set_network_trunk_subports_without_optional_keys(self):
+        subport = copy.copy(self._trunk['sub_ports'][0])
+        # Pop out the segmentation-id and segmentation-type
+        subport.pop('segmentation_type')
+        subport.pop('segmentation_id')
+        arglist = [
+            '--subport', 'port=%(port)s' % {'port': subport['port_id']},
+            self._trunk['name'],
+        ]
+        verifylist = [
+            ('trunk', self._trunk['name']),
+            ('set_subports', [{
+                'port': subport['port_id']}]),
+        ]
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        result = self.cmd.take_action(parsed_args)
+
+        self.neutronclient.trunk_add_subports.assert_called_once_with(
+            self._trunk['name'], {'sub_ports': [subport]}
+        )
+        self.assertIsNone(result)
+
+    def test_set_network_trunk_subports_without_required_key_fail(self):
+        subport = self._trunk['sub_ports'][0]
+        arglist = [
+            '--subport', 'segmentation-type=%(seg_type)s,'
+            'segmentation-id=%(seg_id)s' % {
+                'seg_id': subport['segmentation_id'],
+                'seg_type': subport['segmentation_type']},
+            self._trunk['name'],
+        ]
+        verifylist = [
+            ('trunk', self._trunk['name']),
+            ('set_subports', [{
+                'segmentation-id': str(subport['segmentation_id']),
+                'segmentation-type': subport['segmentation_type']}]),
+        ]
+
+        with testtools.ExpectedException(argparse.ArgumentTypeError):
+            self.check_parser(self.cmd, arglist, verifylist)
+
+        self.neutronclient.trunk_add_subports.assert_not_called()
+
     def test_set_trunk_attrs_with_exception(self):
         arglist = [
             '--name', 'reallylongname',
@@ -621,13 +717,13 @@ class TestUnsetNetworkTrunk(test_fakes.TestNeutronClientOSCV2):
         'sub_ports',
     )
     data = (
-        v2_utils.format_admin_state(_trunk['admin_state_up']),
+        v2_utils.AdminStateColumn(_trunk['admin_state_up']),
         _trunk['id'],
         _trunk['name'],
         _trunk['port_id'],
         _trunk['project_id'],
         _trunk['status'],
-        utils.format_list_of_dicts(_trunk['sub_ports']),
+        format_columns.ListDictColumn(_trunk['sub_ports']),
     )
 
     def setUp(self):
